///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  Copyright  NetworkDLS 2002, All rights reserved
//
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF 
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A 
// PARTICULAR PURPOSE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <Windows.H>
#include <StdIO.H>
#include <StdLib.H>

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "CProcess.H"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CProcess::LoadProcess(const char *sCommandLine)
{
	char sCurrentDirectory[MAX_PATH];

	if(GetCurrentDirectory(sizeof(sCurrentDirectory), sCurrentDirectory))
	{
		return this->LoadProcess(NULL, sCommandLine, sCurrentDirectory);
	}
	else{
		return false;
	}
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CProcess::LoadProcess(const char *sExeName, const char *sParameters)
{
	char sCurrentDirectory[MAX_PATH];

	if(GetCurrentDirectory(sizeof(sCurrentDirectory), sCurrentDirectory))
	{
		return this->LoadProcess(sExeName, sParameters, sCurrentDirectory);
	}
	else{
		return false;
	}
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CProcess::LoadProcess(const char *sExeName, const char *sParameters, const char *sCurrentDirectory)
{
    memset(&this->StartupInfo, 0, sizeof(STARTUPINFO));
    memset(&this->ProcessInfo, 0, sizeof(PROCESS_INFORMATION));

    this->StartupInfo.cb = sizeof(STARTUPINFO);

    BOOL Result = CreateProcess(
        (char *)sExeName,          // pointer to name of executable module
        (char *)sParameters,       // pointer to command line string
        NULL,                      // pointer to process security attributes
        NULL,                      // pointer to thread security attributes
        TRUE,                      // handle inheritance flag
        NORMAL_PRIORITY_CLASS | CREATE_SUSPENDED,  // Creation flags
        NULL,                      // pointer to new environment block
        (char *)sCurrentDirectory, // pointer to current directory name
        &this->StartupInfo,        // pointer to STARTUPINFO
        &this->ProcessInfo         // pointer to PROCESS_INFORMATION
    );

    return(Result > 0);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CProcess::UnloadProcess(void)
{
    return(CloseHandle(this->ProcessInfo.hProcess) && CloseHandle(this->ProcessInfo.hThread) > 0);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CProcess::Pause(void)
{
	return(ResumeThread(this->ProcessInfo.hThread) > 0);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CProcess::Resume(void)
{
	return(ResumeThread(this->ProcessInfo.hThread) > 0);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CProcess::WaitOnExit(void)
{
    DWORD ldwReturnCode = 0;		
	return this->WaitOnExit(&ldwReturnCode);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CProcess::WaitOnExit(DWORD *dwReturnCode)
{
    DWORD ldwReturnCode = 0;		

	do{
        if(!GetExitCodeProcess(this->ProcessInfo.hProcess, &ldwReturnCode))
		{
            return false;
		}
        Sleep(1);
    } while(ldwReturnCode == STILL_ACTIVE);

	*dwReturnCode = ldwReturnCode;

	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CProcess::KillProcess(UINT iExitCode)
{
	return(TerminateProcess(this->ProcessInfo.hProcess, iExitCode) > 0);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CProcess::KillThread(UINT iExitCode)
{
	return(TerminateThread(this->ProcessInfo.hThread, iExitCode) > 0);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

